import 'dart:math';

import 'package:flutter_vasat/models/session.dart';
import 'package:flutter_vasat/models/vasat_model.dart';
import '../vasat_types.dart';

abstract class VasatStorageProvider {

  String dbName = "";
  // For generating random Strings and UUID
  static const _chars = '-AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz1234567890-';
  Random _rnd = Random();

  //const VasatStorageProvider();

  String generateUID(Type classType) {
    return "${classType.toString().toLowerCase()}_${DateTime.now().millisecondsSinceEpoch}_" + getRandomString(20);
  }

  String getRandomString(int length) => String.fromCharCodes(Iterable.generate(length, (_) => _chars.codeUnitAt(_rnd.nextInt(_chars.length))));

  Future<void> openDatabase(String dbName);

  String getObjectKey(String objectName) {
    return dbName + "_" + objectName;
  }

  Future<int?> getObjectLastTimestamp(String vasatObjectName);

  Future<void> setObjectLastTimestamp(String vasatObjectName, int timestamp);

  Future<Session?> getSession();

  Future<bool> saveSession(Session session);

  Future<bool> removeSession();

  Future<bool> saveObjects<T extends VasatModel<T>>(List<T> objects);

  Future<List<T>> getObjects<T extends VasatModel<T>>(VasatFromJsonFactory<T> factory);
}